@@ -1,13 +1,18 @@
+import json
 import logging
 import sys
 from queue import Queue
 from threading import Thread
+
 import pika
 
+from .utils import RBMQ_run_status, RBMQ_aggregation_status
+
 
 class Consumer:
-    def __init__(self, config: dict, msg_queue: Queue):
+    def __init__(self, config: dict, msg_queue: Queue, run_status_callback):
         self.msg_queue = msg_queue
+        self.run_status_callback = run_status_callback
         rabbitmq_config = {"leaf_name": "",
                            "leaf_id": "",
                            "queue_password": "",
@@ -52,11 +57,27 @@ class Consumer:
             return conn
 
     def __callback(self, ch, method, properties, body):
-        msg = body.decode()
-        logging.info("[Consumer] Message received: " + msg)
-
-        self.msg_queue.put(msg, block=False)
-        ch.basic_ack(delivery_tag=method.delivery_tag)
+        try:
+            msg = body.decode()
+            logging.debug("[Consumer] Message received: " + str(msg))
+        except Exception("Error with msg from RabbitMQ") as e:
+            logging.error(f"[Consumer] Failed with msg from RabbitMQ {str(msg)}")
+            raise e
+        try:
+            msg = json.loads(msg)
+        except Exception as e:
+            logging.error(f"[Consumer] Failed decode message to JSON {str(e)}")
+            raise e
+        logging.debug(f"[Consumer] Message type ready {type(msg['message_type'])}")
+        if msg['message_type'] == RBMQ_aggregation_status:
+            logging.debug("[Consumer] Putting agg_id to Queue: " + str(msg['body']['aggregation_id']))
+            self.msg_queue.put(msg['body']['aggregation_id'], block=False)
+            ch.basic_ack(delivery_tag=method.delivery_tag)
+        elif msg['message_type'] == RBMQ_run_status:
+            logging.debug("[Consumer] setting run_status: " + str(msg['body']['run_status']))
+            self.run_status_callback(msg['body']['run_status'])
+        else:
+            logging.error(f"Failed to match message type")
 
     def start(self):
         self.channel.basic_consume(
@@ -74,6 +95,7 @@ class Consumer:
             self.channel.close()
         if self.conn.is_open:
             self.conn.close()
+        return
 
     def spawn_consumer_thread(self):
         t = ConsumerThread(target=self.start, callback=self.stop, daemon=True)
@@ -106,9 +128,8 @@ class ConsumerThread(Thread):
     def localtrace(self, frame, event, arg):
         if self.killed:
             if event == 'line':
-                raise SystemExit()
+                return SystemExit()
         return self.localtrace
 
     def kill(self):
-        # self.callback()
         self.killed = True
